#!/usr/bin/perl
# ------------------------------------------------------------------------------
#  ident "@(#)men_cmm.pl 1.18 02/03/15 SMI"  
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}  

#-------------------------------------------------------------------------------
#
# Management of /etc/services file
#
#-------------------------------------------------------------------------------
sub add_services {
   my ( $services_file, $service_input, $port_entry_input ) = @_ ;
   my $count=0;
   my $service;
   my $port_entry;
   my $garbage;
 
   $count = 0;   
     
   open (SERVICES, $services_file ) || &error("Unable to open $services_file");
   while ( <SERVICES> ) {
      
      chop();
      s/#.*//;
      ($service, $port_entry, $garbage) = split (/ /, $_) ;
      next unless (&defined($service));
      next unless (&defined($port_entry));
            
      if ( "$service" eq "$service_input" ) {
	 &error("Service \"$service\" does not match input \"$port_entry_input\" in $services_file") 
	 unless ( $port_entry eq $port_entry_input );
         $count++;
	 last;
      }
       	        
   }
   close (SERVICES);
   
   unless ( $count ) {
      open (SERVICES, ">>$services_file" ) || &error("Unable to update $services_file");
      print SERVICES "$service_input      $port_entry_input\n";
      close (SERVICES);
   }
   
}


#-------------------------------------------------------------------------------
#
# Configure CMM 
#
#-------------------------------------------------------------------------------
sub cmm_configure {

   my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;

   my $proto_name = build_node_name($proto_node_id, "");
   my $proto_name_nic0 = build_node_name($proto_node_id, "NIC0");
   my $proto_name_cgtp = build_node_name($proto_node_id, "CGTP");
   my $proto_ip   ="${subnet0}.${cluster}.${subnet1}.${proto_node_id}";
   my $server_ip  ="${subnet0}.${cluster}.${subnet1}.${server_node}";
		
   my $local_shared_cmm_conf = "$WORKING_DIR/shared_cmm_conf";
   my $remote_shared_cmm_conf_dir = "/SUNWcgha/local/export/data/etc/";
   my $remote_shared_cmm_conf1 = "${remote_shared_cmm_conf_dir}/cluster_nodes_table";

   my $local_user_cmm_conf = "$WORKING_DIR/user_cmm_conf";
   my $remote_user_cmm_conf_dir = "/etc/opt/SUNWcgha";
   my $remote_user_cmm_conf = "${remote_user_cmm_conf_dir}/nhcmm.conf";
   
   my $local_nodename = "$WORKING_DIR/nodename_${proto_name}";
   my $remote_nodename = "/etc/nodename";
   
   my $node_name_template;	
	
   #
   # If this is the MEN1
   #
   if ( $proto_node_id eq $MEN1_ID ) {
	
     # Create a cluster_nodes_table file
     #
     open(SHARED_CMM_CONFIG, ">$local_shared_cmm_conf") ||
       &error("Unable to create $local_shared_cmm_conf");
	
     print SHARED_CMM_CONFIG "#NodeId\tDomain_id\tName\tAttributes\n";
     
     for $node (0..$#NODE_LIST) {
		
       my $node_id = $NODE_LIST[$node]{id};
       $node_name_template = build_node_name($node_id, "");       
       print SHARED_CMM_CONFIG "${node_id}\t${cluster}\t${node_name_template}\t-\n";
       
     }		
     close(SHARED_CMM_CONFIG);

     # remote mkdir $remote_shared_cmm_conf_dir
     #
     &remote_exec ( "$proto_ip", "$RMKDIR -p $remote_shared_cmm_conf_dir") ||
       &error ( "Unable to create directory $remote_shared_cmm_conf_dir on MEN1");
		
     # remote copy cluster_node_table
     #
     &exec_cmd ("$RCP $local_shared_cmm_conf $proto_ip:$remote_shared_cmm_conf1") ||
       &error ( "Unable to remote copy $local_shared_cmm_conf in MEN1:$remote_shared_cmm_conf1");	

   }

   #
   # Create nhcmm.conf file
   #
   open(USER_CMM_CONFIG, ">$local_user_cmm_conf") ||
     &error("Unable to create $local_user_cmm_conf");
   
   print USER_CMM_CONFIG<<"FEO";
	
# CMM_SHARED_CONFIG_DIR contains location of shared config. file.
CMM_SHARED_CONFIG_DIR=/SUNWcgha/remote/etc
export CMM_SHARED_CONFIG_DIR

# CMM_IS_ELIGIBILITY contains node eligibility (i.e. capacity of becoming a master)
CMM_IS_ELIGIBLE=YES
export CMM_IS_ELIGIBLE

# CMM_DOMAIN_ID contains the cluster id
CMM_DOMAIN_ID=${cluster}
export CMM_DOMAIN_ID

# SET LOCALE 
NLSPATH=/opt/SUNWcgha/lib/locale/C/LC_MESSAGES/%N
export NLSPATH

FEO

      if ($USE_CGTP eq "YES") {
       print USER_CMM_CONFIG<<"FEO";
	
# CMM_NIC0 and CMM_NIC1 contain the two network interfaces for CGTP.
CMM_NIC0=$MEN_NIC0
CMM_NIC1=$MEN_NIC1
export CMM_NIC0
export CMM_NIC1

# CMM_USE_CGTP indicates if cgtp routing has to be enabled
CMM_USE_CGTP=YES
export CMM_USE_CGTP

FEO
      } else {
       print USER_CMM_CONFIG<<"FEO";
	
# CMM_NIC0 and CMM_NIC1 contain the two network interfaces for CGTP.
CMM_NIC0=$MEN_NIC0
CMM_NIC1=notused
export CMM_NIC0
export CMM_NIC1

# CMM_USE_CGTP indicates if cgtp routing has to be enabled
CMM_USE_CGTP=NO
export CMM_USE_CGTP

FEO
      }



   close(USER_CMM_CONFIG);
	
  #
  # remote copy nhcmm.conf
  #
  &exec_cmd ("$RCP $local_user_cmm_conf $proto_ip:$remote_user_cmm_conf") ||
    &error ( "Unable to remote copy $local_user_cmm_conf in $proto_ip:$remote_user_cmm_conf");
	
  #
  # create nodename file
  #
  open(NODENAME, ">${local_nodename}") ||
    &error("Unable to create ${local_nodename}");

  if ($USE_CGTP eq "YES") {
     print NODENAME "${proto_name_cgtp}\n";
  } else {
     print NODENAME "${proto_name_nic0}\n";
  }

  close(NODENAME);
		
  #
  # remote copy nodename
  #
  &exec_cmd ("$RCP $local_nodename $proto_ip:$remote_nodename") ||
    &error ( "Unable to remote copy $local_nodename in $proto_ip:$remote_nodename");						

}


##################################################################################
#
#                               This is the main
#
##################################################################################

init_and_read_arg($0,
	 [ROOT_DIR, PACKAGE_SUBDIR, LOCAL_PACKAGES, NODE_ROLE, WORKING_DIR] , 
	 []) ;

if ($RECOVERY ne "") {

  print_recovery_node("Cluster Membership Monitor (CMM) clean-up") ;
  remove_men_pkg($NODE_ID, $LOCAL_PACKAGES, "LOCAL") ;

} else {

  print_stage_node("Cluster Membership Monitor (CMM) installation") ;
  install_men_pkg ($NODE_ID, $ROOT_DIR, $PACKAGE_SUBDIR, $LOCAL_PACKAGES, "LOCAL", "");

  print_action("CMM configuration");
  cmm_configure ($CLUSTER, $SUBNET0, $SUBNET1, $NODE_ID, $SERVER_NODE);

}

# That's all folk!
exit(0);







