#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)men_cgtp.pl 1.10 02/04/04 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

#-------------------------------------------------------------------------------
#
# Management of /etc/netmasks file
#
#-------------------------------------------------------------------------------
sub add_netmasks {
   my ( $netmasks_file, $address, $maskinput ) = @_ ;
   my $count=0;
   my $ip;
   my $mask;
   my $garbage;
 
   $count = 0;   
     
   open (NETMASKS, $netmasks_file ) || &error("Unable to open $netmasks_file");
   while ( <NETMASKS> ) {
      
      chop();
      s/#.*//;
      ($ip, $mask, $garbage) = split (/ /, $_) ;
      next unless (&defined($ip));
      next unless (&defined($mask));
            
      if ( "$ip" eq "$address" ) {
	 &error("Ip \"$address\" does not match \"$maskinput\" as first hostname in $netmasks_file") 
	 unless ( $mask eq $maskinput );
         $count++;
	 last;
      }
       	        
   }
   close (NETMASKS);
   
   unless ( $count ) {
      open (NETMASKS, ">>$netmasks_file" ) || &error("Unable to update $netmasks_file");
      print  NETMASKS "$address $maskinput\n";
      close (NETMASKS);
   }   
}

#-------------------------------------------------------------------------------
#
# Configure CGTP and/or network 
#
#-------------------------------------------------------------------------------
sub net_configure {

   my ($proto_node_id,$server_node) = @_;

   my $proto_name = build_node_name($proto_node_id, "");
   my $proto_name_if0 = build_node_name($proto_node_id, "NIC0");
   my $proto_name_if1 = build_node_name($proto_node_id, "NIC1");
   my $proto_name_cgtp0 = build_node_name($proto_node_id, "CGTP");	
   my $proto_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${proto_node_id}";
   my $proto_ip_if1 = "${SUBNET0}.${CLUSTER}.${SUBNET2}.${proto_node_id}";
   my $proto_ip_cgtp0 = "${SUBNET0}.${CLUSTER}.${SUBNET3}.${proto_node_id}";
   my $proto_net_if1 = "${SUBNET0}.${CLUSTER}.${SUBNET2}.0";
   my $proto_net_cgtp0 = "${SUBNET0}.${CLUSTER}.${SUBNET3}.0";
   
   my $netmask = "255.255.255.0";

	
   my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${server_node}";
	
   my $hostname_if0 = "/etc/hostname.$MEN_NIC0";
   my $hostname_if1 = "/etc/hostname.$MEN_NIC1";
	
   my $proto_netmasks = "$WORKING_DIR/netmasks_${proto_name}";
   my $proto_hosts = "$WORKING_DIR/hosts_${proto_name}";
	
   #
   # remote create /etc/hostname.<nic0> (already exists but incorrect)
   #
   &remote_exec ( "$proto_ip", "echo $proto_name_if0 > $hostname_if0") || 
     &error ( "Unable to create $hostname_if0" );

   if ($USE_CGTP eq "YES") {
     #
     # remote create /etc/hostname.<nic1>
     #
     &remote_exec ( "$proto_ip", "echo $proto_name_if1 > $hostname_if1") || 
       &error ( "Unable to create $hostname_if1" );
   }

   #
   # retrieve proto_node /etc/netmasks 
   #
   &exec_cmd ( "$RCP $proto_ip:/etc/netmasks $proto_netmasks") ||
     &error ( "Unable to retrieve proto_node /etc/netmasks in $proto_netmasks");
	
   #
   # modify locally proto_node /etc/netmasks 
   #
   # even if CGTP is not used, netmask for nic1 is needed for configuring
   # DHCP. If not defined, the DHCP file (which is required even for NIC1)
   # is created with a wrong name
   &add_netmasks( $proto_netmasks, $proto_net_if1, $netmask );

   if ($USE_CGTP eq "YES") {
     &add_netmasks( $proto_netmasks, $proto_net_cgtp0, $netmask );
   }
	
   #
   # putback modified proto_node /etc/netmasks
   #
   &exec_cmd( "$RCP $proto_netmasks $proto_ip:/etc/netmasks" ) ||
     &error ( "Unable to putback $proto_netmasks on $proto_ip:/etc/netmasks") ;
	
   #
   # retrieve /etc/hosts
   #
   &exec_cmd( "$RCP $proto_ip:/etc/hosts $proto_hosts" ) ||
     &error ( "Unable to retrieve $proto_hosts from $proto_ip:/etc/hosts") ;	

   #
   # create locally proto_node /etc/hosts 
   #
   update_hosts_file($proto_hosts, $proto_node_id) ;

   #
   # putback modified proto_node /etc/hosts
   #
   &exec_cmd( "$RCP $proto_hosts $proto_ip:/etc/hosts" ) ||
     &error ( "Unable to putback $proto_hosts on $proto_ip:/etc/hosts") ;	
	
}

##################################################################################
#
#                               This is the main
#
##################################################################################
    
init_and_read_arg($0,
		  [ROOT_DIR, PACKAGE_SUBDIR, LOCAL_PACKAGES, NODE_ROLE, WORKING_DIR],
		  []);

if ($USE_CGTP eq "YES") {
  print_stage_node("Carrier-Grade Transport Protocol (CGTP) installation") ;
  install_men_pkg($NODE_ID, $ROOT_DIR, $PACKAGE_SUBDIR, $LOCAL_PACKAGES, "LOCAL", "") ;

  print_action("CGTP configuration") ;
  net_configure ($NODE_ID, $SERVER_NODE);

} else {
  print_stage_node("Network configuration") ;
  net_configure ($NODE_ID, $SERVER_NODE);

}
# That's all folk!
exit(0);







