#!/usr/bin/perl
# ------------------------------------------------------------------------------
#  ident "@(#)men_svm.pl 1.1 02/04/03 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

#--------------------------------------------------------------------------------
#
# creation of metadevice
#
#--------------------------------------------------------------------------------

sub configure_svm() {

  my $node_ip = "$SUBNET0.$CLUSTER.$SUBNET1.$NODE_ID";

  # creation of state database replica

  print_action("Creating metadevice database");
  remote_exec($node_ip, "$RMETADB -a -f $SLICE_REPLICA") || 
    &error ( "Unable to create meta database on slice $SLICE_REPLICA");

  # creation of each concat metadevice
  for my $i (0..$#SLICE_LIST) {
    if ($SLICE_LIST[$i]{metadevice} ne "") {
      print_action("converting slice $SLICE_LIST[$i]{name} into metadevice $SLICE_LIST[$i]{metadevice}");

      # unmount UFS partition if mounted
      # ignore error, it may stay unmount after a recovery
      if ($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED) {
	print_subaction("umount $SLICE_LIST[$i]{mounting_point}");
	remote_exec_ignore($node_ip, "$RUMOUNT $SLICE_LIST[$i]{mounting_point}") ;
      }

      # create the metadevice
      print_subaction("creating metadevice $SLICE_LIST[$i]{metadevice}");
      remote_exec($node_ip, "$RMETAINIT $SLICE_LIST[$i]{metadevice} 1 1 $SLICE_LIST[$i]{name}") ||
	&error ( "Unable to create metadevice $SLICE_LIST[$i]{metadevice} on $SLICE_LIST[$i]{name}" );

      #create the filesystem on the raw partition
      if ($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED) {
	print_subaction("creating filesystem on $SLICE_LIST[$i]{raw}");
	remote_exec($node_ip, "$RNEWFS $SLICE_LIST[$i]{raw}") ||
	  &error ( "Unable to create the filesystem on $SLICE_LIST[$i]{raw}");	
      }
    }
  }

}
#--------------------------------------------------------------------------------
#
# update /etc/vfstab to replace physical slice name by metadevice name
#
#--------------------------------------------------------------------------------

sub update_vfstab_entry {
  my ($vfstab_file, $slice, $metadevice_block, $metadevice_raw ) = @_ ;

  my $current_block;
  my $current_raw;
  my $garbage1;
  my $garbage2;
  my $garbage3;
  my $garbage4;
  my $garbage5;
    
  open (VFSTAB, $vfstab_file ) || &error("Unable to open $vfstab_file");
  open (VFSTAB_TMP, ">${vfstab_file}.tmp" ) || &error("Unable to open ${vfstab_file}.tmp");
  
  while ( <VFSTAB> ) {
    
    chop();
    s/#.*//;
    ($current_block, $current_raw, $garbage1, $garbage2, $garbage3, $garbage4, $garbage5) =
      split (/\t/, $_) ;
    if ( $current_block eq "/dev/dsk/$slice" ) {
      print VFSTAB_TMP "$metadevice_block\t$metadevice_raw\t$garbage1\t$garbage2\t$garbage3\t$garbage4\t$garbage5\n";
      $count++;
    } else {
      print VFSTAB_TMP "$_\n";
    }
    next;       	        
  }
   
  close (VFSTAB);
  close (VFSTAB_TMP);
	
	
  exec_cmd ( "$MV ${vfstab_file}.tmp ${vfstab_file}") ||
    error ( "Unable to move ${vfstab_file}.tmp ${vfstab_file}");  
}

#--------------------------------------------------------------------------------
#
# update /etc/vfstab to replace physical slice name by metadevice name
#
#--------------------------------------------------------------------------------

sub update_vfstab {
  my $node_ip = "$SUBNET0.$CLUSTER.$SUBNET1.$NODE_ID";
  
  my $local_vfstab = "$WORKING_DIR/vfstab";
  my $remote_vfstab = "/etc/vfstab";
  
  print_action("Updating /etc/vfstab");
  #
  # retrieve /etc/vfstab file
  #
  exec_cmd ( "$RCP $node_ip:$remote_vfstab $local_vfstab") ||
    error ( "Unable to retrieve remote $remote_vfstab in $local_vfstab");
  
  for my $i (0..$#SLICE_LIST) {
    if (($SLICE_LIST[$i]{metadevice} ne "") && ($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED)) {
      print_action("updating entry for $SLICE_LIST[$i]{name}");
      update_vfstab_entry($local_vfstab, 
			  $SLICE_LIST[$i]{name},
			  $SLICE_LIST[$i]{block},
			  $SLICE_LIST[$i]{raw});
    }
  }
  
  #
  # putback modified /etc/vfstab
  #
  exec_cmd( "$RCP $local_vfstab $node_ip:${remote_vfstab}" ) ||
    error ( "Unable to putback $local_vfstab on  remote${remote_vfstab}") ;
}

#--------------------------------------------------------------------------------
#
# mount newly created slices
#
#--------------------------------------------------------------------------------

sub mount_slice {
  my $node_ip = "$SUBNET0.$CLUSTER.$SUBNET1.$NODE_ID";
  
  print_action("Mounting new slices") ;
  for my $i (0..$#SLICE_LIST) {
    if (($SLICE_LIST[$i]{metadevice} ne "") && ($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED)) {
      print_subaction("mounting $SLICE_LIST[$i]{mounting_point} on $SLICE_LIST[$i]{block}");
      remote_exec($node_ip, "$RMOUNT $SLICE_LIST[$i]{mounting_point}") ||
	&error ( "Unable to mount $SLICE_LIST[$i]{mounting_point}");
    }
  }
}

#--------------------------------------------------------------------------------
#
# undo for recovery
#
#--------------------------------------------------------------------------------

sub undo_svm {

  my $node_ip = "$SUBNET0.$CLUSTER.$SUBNET1.$NODE_ID";

  # remove metadevice definition
  for my $i (0..$#SLICE_LIST) {
    if ($SLICE_LIST[$i]{metadevice} ne "") {

      print_action("removing slice $SLICE_LIST[$i]{name} adaptation");

      # unmount partition if mounted
      # ignore error, it may be not mounted
      if ($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED) {
	print_subaction("umount $SLICE_LIST[$i]{mounting_point}");
	remote_exec_ignore($node_ip, "$RUMOUNT $SLICE_LIST[$i]{mounting_point}") ;
      }

      # remove the metadevice partition
      print_subaction("remove metadevice $SLICE_LIST[$i]{metadevice}") ;
      remote_exec_ignore($node_ip, "$RMETACLEAR $SLICE_LIST[$i]{metadevice}");
    }
  }

  # remove the meta database
  remote_exec_ignore($node_ip, "$RMETADB -f -d $SLICE_REPLICA") ;
  
}

##################################################################################
#
#                               This is the main
#
##################################################################################

@required = [ NODE_ROLE, ROOT_DIR, WORKING_DIR] ;
@optional = [  ] ;
init_and_read_arg($0, @required, @optional) ;

if ($USE_SVM ne "YES") {
  exit(0);
}

if ($RECOVERY ne "") {
  print_recovery_node("Disksuite configuration clean-up");
  undo_svm();
  
} else {
  
  print_stage_node("DiskSuite configuration");
  configure_svm();
  update_vfstab();
  mount_slice();
}

# That's all folk!
exit(0);







