#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)nmen_cgtp.pl 1.14 02/03/15 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}
    
#-------------------------------------------------------------------------------
#
# Managemnt of /etc/netmasks file
#
#-------------------------------------------------------------------------------
sub add_netmasks {
   my ( $netmasks_file, $address, $maskinput ) = @_ ;
   my $count=0;
   my $ip;
   my $mask;
   my $garbage;
 
   $count = 0;   
     
   open (NETMASKS, $netmasks_file ) || &error("Unable to open $netmasks_file");
   while ( <NETMASKS> ) {
      
      chop();
      s/#.*//;
      ($ip, $mask, $garbage) = split (/ /, $_) ;
      next unless (&defined($ip));
      next unless (&defined($mask));
            
      if ( "$ip" eq "$address" ) {
	 &error("Ip \"$address\" does not match \"$maskinput\" as first hostname in $netmasks_file") 
	 unless ( $mask eq $maskinput );
         $count++;
	 last;
      }
       	        
   }
   close (NETMASKS);
   
   unless ( $count ) {
      open (NETMASKS, ">>$netmasks_file" ) || &error("Unable to update $netmasks_file");
      print  NETMASKS "$address $maskinput\n";
      close (NETMASKS);
   }   
}

#-------------------------------------------------------------------------------
#
# Configure CGTP and/or network
#
#-------------------------------------------------------------------------------
sub net_configure {

   my ($proto_node_id,$server_node) = @_;

   my $proto_name = build_node_name($proto_node_id, "");
   my $proto_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${proto_node_id}";
   my $proto_net_if0 = "${SUBNET0}.${CLUSTER}.${SUBNET1}.0";
   my $proto_net_if1 = "${SUBNET0}.${CLUSTER}.${SUBNET2}.0";
   my $proto_net_cgtp0 = "${SUBNET0}.${CLUSTER}.${SUBNET3}.0";
		
   my $netmask = "255.255.255.0";
	
   my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${server_node}";
	
   my $hostname_if0;
   my $hostname_if1;
   my $proto_hosts = "$WORKING_DIR/hosts_${proto_name}";
	
   my $proto_netmasks;
   
   my $node_name_template;
   my $node_name_if0_template;
   my $node_name_if1_template;
   my $node_name_cgtp0_template;
   
   my $node_root;


   for my $node (0..$#NODE_LIST) {
     
     if ($node ne $MEN1 && $node ne $MEN2) {
					
       my $node_id = $NODE_LIST[$node]{id};

       $node_name_template = build_node_name($node_id, "");			
       $node_name_if0_template = build_node_name($node_id, "NIC0");			
       $node_name_if1_template = build_node_name($node_id, "NIC1");
       $node_name_cgtp0_template= build_node_name($node_id, "CGTP");
			
       $node_root = "/export/root/$node_name_template";
			
       $proto_netmasks = "$WORKING_DIR/netmasks_${node_name_template}";
		
       $hostname_if0 = "/etc/hostname.$NODE_LIST[$node]{nic0}";
       $hostname_if1 = "/etc/hostname.$NODE_LIST[$node]{nic1}";
       #
       # remote create diskless node /etc/hostname.<nic0>
       #
       &remote_exec ( "$proto_ip", "echo $node_name_if0_template > $node_root/$hostname_if0") || 
	 &error ( "Unable to create $node_root/$hostname_if0" );
					
       #
       # remote create diskless node /etc/hostname.<nic1>
       #
       if ($USE_CGTP eq "YES") {
	 &remote_exec ( "$proto_ip", "echo $node_name_if1_template > $node_root/$hostname_if1") || 
	   &error ( "Unable to create $node_root/$hostname_if1" );
       }

       #
       # retrieve diskless node /etc/netmasks 
       #
       &exec_cmd ( "$RCP $proto_ip:$node_root/etc/netmasks $proto_netmasks") ||
	 &error ( "Unable to retrieve proto_node $node_root/etc/netmasks in $proto_netmasks");
	
       #
       # modify locally proto_node /etc/netmasks 
       #
       &add_netmasks( $proto_netmasks, $proto_net_if0, $netmask );
       if ($USE_CGTP eq "YES") {
	 &add_netmasks( $proto_netmasks, $proto_net_if1, $netmask );
	 &add_netmasks( $proto_netmasks, $proto_net_cgtp0, $netmask );
       }

       #
       # putback modified proto_node /etc/netmasks
       #
       &exec_cmd( "$RCP $proto_netmasks $proto_ip:$node_root/etc/netmasks" ) ||
	 &error ( "Unable to putback $proto_netmasks on proto_node:$node_root/etc/netmasks") ;
       
       #
       # create /etc/hosts
       #
       create_hosts_file($proto_hosts, $node_id) ;
       &exec_cmd( "$RCP $proto_hosts $proto_ip:$node_root/etc/hosts" ) ||
	 &error ( "Unable to putback $proto_hosts on proto_node:$node_root/etc/hosts") ;
     }		
   }
}

			

##################################################################################
#
#                               This is the main
#
##################################################################################

init_and_read_arg($0,
		  [ROOT_DIR, PACKAGE_SUBDIR, LOCAL_PACKAGES, NODE_ROLE, WORKING_DIR],
		  []);

if ($USE_CGTP eq "YES") {

  print_stage_node("Carrier-Grade Transport Protocol (CGTP) installation for diskless") ;
  install_nmen_pkg($NODE_ID, $ROOT_DIR, $PACKAGE_SUBDIR, $LOCAL_PACKAGES, "");

  print_action("CGTP configuration") ;
  net_configure ($NODE_ID, $SERVER_NODE);

} else {

  print_stage_node("Network configuration for diskless") ;

  net_configure ($NODE_ID, $SERVER_NODE);

}

# That's all folk!
exit(0);







