#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)nmen_diskless.pl 1.6 02/04/11 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

#------------------------------------------------------------------------------
#
# end_vice_master_diskless_env :
# - copy /etc/ethers from master to this node
# - complete /etc/bootparmas with diskless entries
#------------------------------------------------------------------------------

sub end_vice_master_diskless_env {

   my ($cluster,$subnet0,$subnet1,$master_node_id, $vice_master_node_id, $server_node) = @_;

   my $vice_master_name = build_node_name($vice_master_node_id, "");
   my $master_name= build_node_name($master_node_id, "");

   my $vice_master_ip ="${subnet0}.${cluster}.${subnet1}.${vice_master_node_id}";
   my $master_ip ="${subnet0}.${cluster}.${subnet1}.${master_node_id}";

   my $local_bootparams ="$WORKING_DIR/bootparams_${master_name}";
   my $remote_bootparams = "/etc/bootparams";

   my $ethers_file ="/etc/ethers";
   my $local_ethers="$WORKING_DIR/ethers.tmp";

   if (scalar($NODE_LIST) > 2) {
     # if there is no diskless defined, the file doesn't exist
     exec_cmd("$RCP ${master_ip}:${ethers_file} $local_ethers") ||
       error("Unable to copy ${master_ip}:${ethers_file} to $local_ethers");
     exec_cmd("$RCP $local_ethers ${vice_master_ip}:${ethers_file}") ||
       error("Unable to copy $local_ethers to ${vice_master_ip}:${ethers_file}");
   }

   # get localy from master node the /etc/bootparams file and update it to have 
   # second node id in it.
   
   &exec_cmd("$RCP ${master_ip}:${remote_bootparams} ${local_bootparams}") ||
     &error("Cannot copy ${remote_bootparams} from ${master_ip}");
   
   open(BOOTPARAMS,"${local_bootparams}") || &error("Unable to create ${local_bootparams}");
   open(TMP_FILE,">${local_bootparams}.tmp") ||
     &error("Unable to create ${local_bootparams}");
   
   while ( <BOOTPARAMS> ) {	
     chop();
     s/${master_name}/${vice_master_name}/g;
     print TMP_FILE "$_\n";
   }
   close(BOOTPARAMS);
   close(TMP_FILE);
   
   #
   # remote copy bootparams to second node
   #
   
   &exec_cmd ("$RCP ${local_bootparams}.tmp ${vice_master_ip}:${remote_bootparams}") ||
     &error ( "Unable to remote copy ${local_bootparams}.tmp in ${vice_master_ip}:${remote_bootparams}");
   
}

#-------------------------------------------------------------------------------
#
# Declare Solaris diskless client on MEN1
#
#-------------------------------------------------------------------------------
sub smdiskless {

   my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;
   my $proto_name = build_node_name($proto_node_id, "");
   my $proto_ip   ="${subnet0}.${cluster}.${subnet1}.${proto_node_id}";
	
   my $server_ip  ="${subnet0}.${cluster}.${subnet1}.${server_node}";
   my $node_ip_template;
   my $node_eth_template;
   my $node_eth;
   my $node_name_template;
   my $cmd;
   my $node_ip_MEN2;
   my $i=1;
   my $crfs_file="/etc/opt/SUNWcgha/crfs.share";
   my $local_crfs="$WORKING_DIR/crfs.tmp";
   my $dfstab_file="/etc/dfs/dfstab";
   my $local_dfstab="$WORKING_DIR/dfstab.tmp";
   my $setting_file="/etc/default/init" ;

   #
   # We issue the smdiskless for all diskless nodes
   #
   for my $node (0..$#NODE_LIST) {
     
     my $node_id = $NODE_LIST[$node]{id};

     $node_ip_template = "${subnet0}.${cluster}.${subnet1}.${node_id}";
     $node_eth = $NODE_LIST[$node]{eth0};
     if ($node_eth eq "") {
       # MAC address not configured: DHCP mode is dynamic but argument is required for smdiskless
       # depends on the node rank to have a different MAC address per diskless
       $hexa = sprintf("%2.2x", $node_id);
       $node_eth = "00:00:00:00:00:$hexa" ;
     }

     if ($node ne $MEN1 && $node ne $MEN2) {
       $node_name_template = build_node_name($node_id, "");

       print_subaction("adding $node_name_template") ;

       $cmd = "$RSMDISKLESS add -p sunrules -- -i ${node_ip_template} -e $node_eth -n ${node_name_template} -x os=sparc.${ARCH}.Solaris_8";													
       &remote_exec ( "$proto_ip", $cmd) ||
	 &error( " Cannot remotely execute $cmd" );

       # we copy the /etc/default/init to set the timezone and other variables
       &remote_exec ( "$proto_ip", "$RCOPY $setting_file /export/root/${node_name_template}${setting_file}" ) || 
	 &error ( "Unable to cp $setting_file into /export/root/${node_name_template}${setting_file}" );
					
     }
     $i++;
   }

   # Then we update crfs.share to make diskless directories managed by crfs
   if ($ENV{OPTIMIZED_INSTALL} eq "YES") {
     # crfs.share has been updated but its contents has not been copied into dfstab
     # (RNFS not started)
     # files must be concatenated
     &remote_exec ( "$proto_ip", "$RCAT $crfs_file $dfstab_file > $crfs_file.tmp" ) || 
       &error ( "Unable to cat $dfstab_file and $crfs_file on $proto_name" );
     &remote_exec ( "$proto_ip", "$RMV $crfs_file.tmp $crfs_file" ) || 
       &error ( "Unable to mv $crfs_file.tmp to $crfs_file on $proto_name" );
   } else {
     # previous reboot has copied crfs.share into dfstab  (RNFS started)
     # so just copy dfstab to crfs.share
     &remote_exec ( "$proto_ip", "$RCOPY $dfstab_file $crfs_file" ) || 
       &error ( "Unable to cp $dfstab_file into $crfs_file on $proto_name" );
   }

   #
   # dfstab from the second is copied on the installation node
   # this avoid same lines to appear in dfstab and crfs.share on the installation node
   #
   $node_ip_MEN2 = "${subnet0}.${cluster}.${subnet1}.${MEN2_ID}";

   exec_cmd("$RCP ${node_ip_MEN2}:$dfstab_file $local_dfstab") ||
     error ( "Unable to copy ${node_ip_MEN2}:$dfstab_file to $dfstab_file " );
   exec_cmd("$RCP $local_dfstab $proto_ip:$dfstab_file") ||
     error ( "Unable to copy $local_dfstab to $proto_ip:$dfstab_file" );

   #
   # We update the MEN2 crfs.share
   #

   exec_cmd("$RCP $proto_ip:$crfs_file $local_crfs") ||
     error ( "Unable to copy $proto_ip:$crfs_file to $local_crfs" );
   exec_cmd("$RCP $local_crfs ${node_ip_MEN2}:$crfs_file") ||
     error ( "Unable to copy $local_crfs to ${node_ip_MEN2}:$crfs_file" );
}

#-------------------------------------------------------------------------------
#
# Clean up the diskless environment
#
#-------------------------------------------------------------------------------
sub smdiskless_cleanup {

   my ($node_id) = @_;
   my $node_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${node_id}";

   #
   # remove root and swap directory for each diksless
   #
   for $node (0..$#NODE_LIST) {
     if ($node ne $MEN1 && $node ne $MEN2) {

       $node_name_template = build_node_name($NODE_LIST[$node]{id}, "");

       print_subaction("cleaning $node_name_template diskless environment") ;
       remote_exec ( "$node_ip", "$RRM -Rf /export/root/$node_name_template") ;
       remote_exec ( "$node_ip", "$RRM -Rf /export/swap/$node_name_template") ;
		    	
     }
   }
}

##################################################################################
#
#                               This is the main
#
##################################################################################

init_and_read_arg($0,
		  [ NODE_ROLE, WORKING_DIR],
		  []);


if ($RECOVERY ne "") {
  if ($NODE_ID eq $MEN1_ID) {
    print_recovery_node("Diskless environment clean-up") ;
    smdiskless_cleanup ($NODE_ID) ;
  }
} else {
  if  ( $NODE_ID eq $MEN1_ID ) {
    print_stage_node("Diskless environment creation") ;
    smdiskless($CLUSTER, $SUBNET0, $SUBNET1, $NODE_ID , $SERVER_NODE);
  } else {
    
    # Complete the diskless environment on the second eligible master node
    # copy /etc/ethers from MEN1
    print_stage_node("Completing the vice-master node diskless environment");
    end_vice_master_diskless_env($CLUSTER, $SUBNET0, $SUBNET1, $MEN1_ID, $NODE_ID, $SERVER_NODE);
  }
}

# That's all folk!
exit(0);







