#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)nmen_solaris.pl 1.26 02/04/11 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}

#-------------------------------------------------------------------------------
#
# Set up OS services
#
#-------------------------------------------------------------------------------
sub smosservice {

   my ($cluster,$subnet0,$subnet1,$proto_node_id,$server_node) = @_;
   my $proto_name = build_node_name($proto_node_id, "");
   my $proto_name_cgtp = build_node_name($proto_node_id, "CGTP");
   my $proto_name_if0 = build_node_name($proto_node_id, "NIC0");
   my $proto_ip   ="${subnet0}.${cluster}.${subnet1}.${proto_node_id}";
   my $server_ip  ="${subnet0}.${cluster}.${subnet1}.${server_node}";
   my $Solaris_file ="/var/sadm/system/admin/services/Solaris_8";
   my $local_solaris="$WORKING_DIR/solaris.tmp";
   my $local_bootparams ="$WORKING_DIR/bootparams_${proto_name}";
   my $remote_bootparams = "/etc/bootparams";
   my $idx;
   
   #
   # Test the SOLARIS_DIR directory tree
   #
   &control_dir_is_readable ("$SOLARIS_DIR");

   #
   # remotely create /os (mandatory for smosservice Solaris mount point (bug))
   #
   &trace(1, "remotely create /os on $proto_ip");
   &remote_exec ( "$proto_ip", "$RMKDIR -p /os" ) ||
     &error ( "Unable to create directory /os" );
	
   #
   # mount SOLARIS_DIR
   #   
   remote_mount($proto_ip, $server_ip, $SOLARIS_DIR, "/os") ;

   #
   # If it is MEN1 (master) : remotely execute smosservice
   #
   #
   # If it is MEN2 (vice-master) : issue 15 : retrieve /var/sadm/system/admin/services/Solaris_8 
   # from MEN1 and create /etc/bootparams
   #
   if ( $proto_node_id eq $MEN1_ID ) {

     # execute smossservices
     &remote_exec ( "$proto_ip", "$RSMOSSERVICE add -p sunrules -- -x mediapath=/os -x platform=sparc.$ARCH.Solaris_8 -x cluster=SUNWCuser -x locale=en_US" ) ||
       &error ( "Unable to execute $RSMOSSERVICE add -p sunrules -- -x mediapath=/os -x platform=sparc.$ARCH.Solaris_8 -x cluster=SUNWCuser -x locale=en_US" );

   } else {
     #
     # retrieve /var/sadm/system/admin/services/Solaris_8 (copy from MEN1 on MEN2)
     #
     exec_cmd("$RCP ${subnet0}.${cluster}.${subnet1}.${MEN1_ID}:${Solaris_file} $local_solaris") ||
       error ( "Unable to copy ${subnet0}.${cluster}.${subnet1}.${MEN1_ID}:${Solaris_file} on $local_solaris");
     exec_cmd("$RCP $local_solaris $proto_ip:${Solaris_file}") ||
       error ( "Unable to copy $local_solaris on $proto_ip:${Solaris_file}");
       		
     #
     # create bootparams file
     #
     open(BOOTPARAMS, ">${local_bootparams}") ||
       &error("Unable to create ${local_bootparams}");
     if ($USE_CGTP eq "YES") {
       print BOOTPARAMS "${proto_name_cgtp} boottype=:os\n";	
     } else {
       print BOOTPARAMS "${proto_name_if0} boottype=:os\n";	
     }
     close(BOOTPARAMS);	
	
     #
     # remote copy bootparams
     #
     &exec_cmd ("$RCP $local_bootparams $proto_ip:$remote_bootparams") ||
       &error ( "Unable to remote copy $local_bootparams in $proto_ip:$remote_bootparams");
   }
	
   #
   # umount SOLARIS_DIR
   #   
   &remote_exec ( "$proto_ip", "$RUMOUNT /os" ) || 
     &error ( "Unable to NFS umount /os" );	
 }

#-------------------------------------------------------------------------------
#
# Remove OS services
#
#-------------------------------------------------------------------------------
sub smosservice_cleanup {

   my ($node_id) = @_;
   my $node_ip   ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${node_id}";
   
   my $loc = "/export/Solaris_8" ;
   if ($TARGET_OS eq "Solaris_9") {
     $loc = "/export/Solaris_9" ;
   }

   remote_exec("$node_ip", "$RRM -Rf $loc") ;
   remote_exec("$node_ip", "$RRM -Rf /export/root/clone") ;
 }

##################################################################################
#
#                               This is the main
#
##################################################################################

init_and_read_arg($0,
		  [ SOLARIS_DIR, NODE_ROLE, WORKING_DIR],
		  []);


#--------------------------------------------------------------------------------#
#
# Use absolute pathname for directories
#
#--------------------------------------------------------------------------------#

if ( &defined($SOLARIS_DIR) ) {
   $SOLARIS_DIR = "$EXECDIR"."/".$SOLARIS_DIR unless ( $SOLARIS_DIR =~ /^\// );
}

if ($RECOVERY ne "") {
  if ($NODE_ID eq $MEN1_ID) {
    print_recovery_node("Solaris OS services for diskless clean-up") ;
    smosservice_cleanup($NODE_ID) ;
  }
} else {
  if ($NODE_ID eq $MEN1_ID) {

    print_stage_node("Solaris OS services installation for diskless") ;
    my $duration = 40 ;
    if ($ENV{OPTIMIZED_INSTALL} eq "YES") {
      $duration = 20;
    }
    print_action("this operation may take about $duration minutes to complete") ;

    # they must be mounted: it's the first operation after reboot (in optimized install)
    mount_nmen($NODE_ID);

  } else {
    print_stage_node("Bootparams configuration for diskless") ;
  }

  smosservice($CLUSTER, $SUBNET0, $SUBNET1, $NODE_ID , $SERVER_NODE);
}

# That's all folk!
exit(0);
