#!/usr/bin/perl
# ------------------------------------------------------------------------------
#   ident "@(#)addon.pl 1.3 02/05/03 SMI"
# ------------------------------------------------------------------------------
#
#  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
# ------------------------------------------------------------------------------
package main;

BEGIN {
	require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
}


#
# What and when ?
#
#
# Packages and patches installed during a call to this script depend on:
# . the phase assigned to the package or the patch.
# . the scope of the package
# . the current phase
#
# Herefafter actions performed:
#
# Scope LOCAL:
#
# S (after Solaris)
# . installation of elements for MEN if current phase is S_MEN
# . installation of elements for NMEN if node=1, if current phase is S_NMEN
# F (after Foundation Services)
# . installation of elements for MEN if current phase is F_MEN
# . installation of elements for NMEN if node=1, if current phase is F_NMEN
# D (after Foundation Services on diskless)
# . installation of elements for MEN for both nodes and for NMEN if node=1 if current phase is F_NMEN
# A (after Application Services)
# . installation of elements for MEN if current phase is A_MEN and node is not master node reference
#   explanation:
#   . if node is the master node reference:
#     means that the script is called to install shared element: do not install local elements (see scope SHARED)
#   . if node is 1 or 2:
#     installed local elements.
# . installation of elements for NMEN if current phase is A_NMEN
#
#
# Scope SHARED 
#
# F
# . installation of elements if current phase is F_MEN and node=1
# A
# . installation of element if current phase is A_MEN and node is master node reference
#
#
# Scope USR_SPECIFIC
#
# S (after Solaris)
# . installation of elements for MEN if current phase is S_MEN
# . installation of elements for MEN on /export/Solaris_8 if node=1, if current phase is S_NMEN


my $FOR_MEN="";
my $FOR_NMEN="";
my $WHERE="";

my $SCOPE; # to_be_processed can change the scope defined by the user
my $CHANGE; # indicate that a change has been made and a warning must be displayed

#
#
#-----------------------------------------------------------
#
#   determine if the patch must be installed
#
#-----------------------------------------------------------

sub to_be_processed
  {
    my ($phase, $men, $nmen, $scope) = @_;

    my $installed = FALSE ;
    $SCOPE = $scope;
    $CHANGE = "NO";

    $FOR_MEN = FALSE ;
    $FOR_NMEN = FALSE ;

    if ($scope eq "SHARED") {
      if (($phase eq "F") && ($PHASE eq "F_MEN") && ($NODE_ID eq $MEN1_ID)) {
	$FOR_MEN = TRUE;
	return TRUE ;
      } 
      if (($phase eq "A") && ($PHASE eq "A_MEN") && ($NODE_ID eq $MASTER_ID)) {
	$FOR_MEN = TRUE;
	return TRUE;
      }
      return FALSE;
    }

    if ($scope eq "USR_SPECIFIC") {
      if (($phase eq "S") && ($PHASE eq "S_MEN") && ($men eq "Y")) {
	$FOR_MEN = TRUE;
	# in that case, the scope is changed to LOCAL (/usr is on root file system)
	$SCOPE="LOCAL";
	$CHANGE="YES";
	return TRUE ;
      } 
      if (($phase eq "S") && ($PHASE eq "S_NMEN") && ($nmen eq "Y") && ($NODE_ID eq $MEN1_ID)) {
	# it's for NMEN but install on MEN (/export/Solaris_8)
	$FOR_MEN = TRUE;
	return TRUE;
      }
      return FALSE;
    }

  SWITCH:
      {
	if ($PHASE eq "S_MEN") {
	  if (($phase eq "S") && ($men eq "Y")) {
	    $installed = TRUE ;
	    $FOR_MEN = TRUE;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "F_MEN") {
	  if (($phase eq "F") && ($men eq "Y")) {
	    $installed = TRUE ;
	    $FOR_MEN = TRUE;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "S_NMEN") {
	  # only on node 1 (master during foundation services installation
	  if (($phase eq "S") && ($nmen eq "Y") && ($NODE_ID eq $MEN1_ID)) {
	    $installed = TRUE ;
	    $FOR_NMEN = TRUE;
	    $WHERE = "SOLARIS" ;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "F_NMEN") {
	  # only on node 1 (master during foundation services installation
	  if (($phase eq "F") && ($nmen eq "Y") && ($NODE_ID eq $MEN1_ID)) {
	    $installed = TRUE ;
	    $FOR_NMEN = TRUE;
	    $WHERE = "DISKLESS" ;
	    last SWITCH;
	  }
	  if ($phase eq "D") {
	    $installed = TRUE ;
	    if ($men eq "Y") {
	      $FOR_MEN = TRUE;
	    }
	    if (($nmen eq "Y") && ($NODE_ID eq $MEN1_ID)) {
	      $FOR_NMEN = TRUE;
	      $WHERE = "DISKLESS" ;
	    }
	    last SWITCH;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "A_MEN") {
	  if (($phase eq "A") && ($men eq "Y") && ($NODE_ID ne $MASTER_ID)) {
	    $installed = TRUE ;
	    $FOR_MEN = TRUE;
	  }
	  last SWITCH;
	}
	
	if ($PHASE eq "A_NMEN") {
	  # only on master node
	  if (($phase eq "A") && ($nmen eq "Y")) {
	    $installed = TRUE ;
	    $FOR_NMEN = TRUE;
	    $WHERE = "DISKLESS" ;
	  }
	  last SWITCH;
	}   
      }

    return $installed ;
  }

#-----------------------------------------------------------
# PACKAGES
#-----------------------------------------------------------

#-----------------------------------------------------------
#
#   installing a list of packages
#
#-----------------------------------------------------------

sub install_packages
  {
    my $node_ip    ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${NODE_ID}";
    my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${SERVER_NODE}";

    my $pkgdir = "";
    my $pkgsubdir = "";

    for (my $i = 0 ; $i < scalar(@PACKAGE_LIST) ; $i++) {
      if (to_be_processed($PACKAGE_LIST[$i]{phase}, $PACKAGE_LIST[$i]{men},
			  $PACKAGE_LIST[$i]{nmen}, $PACKAGE_LIST[$i]{scope}) eq TRUE) 
	{
	  if ($CHANGE eq "YES") {
	    print_log("scope $PACKAGE_LIST[$i]{scope} changed to $SCOPE for $PACKAGE_LIST[$i]{reference} when installing on MEN\n");
	  }
	  if ($PACKAGE_LIST[$i]{dir} eq "") {
	    # the default ROOT_DIR is used
	    $pkgdir = $SOLARIS_DIR ;
	  } else {
	    $pkgdir = $PACKAGE_LIST[$i]{dir};
	    exec_cmd("$SHARE -F nfs -o ro,anon=0 $pkgdir") ||
	      error ( "Unable to NFS share $pkgdir" );
	  }

	  if (($PACKAGE_LIST[$i]{subdir} eq "") 
	      and ($PACKAGE_LIST[$i]{dir} eq "")) {
	    # the default SOLARIS_DIR is used
	    $pkgsubdir = $SOLARIS_SUBDIR ;
	  } else {
	    $pkgsubdir = $PACKAGE_LIST[$i]{subdir};
	  }

	  if ($FOR_MEN eq TRUE) {
	    install_men_pkg($NODE_ID, $pkgdir, $pkgsubdir,
			      $PACKAGE_LIST[$i]{reference}, $SCOPE, "") ;
	  } 
	  if ($FOR_NMEN eq TRUE) {
	    install_nmen_pkg($NODE_ID, $pkgdir, $pkgsubdir,
			      $PACKAGE_LIST[$i]{reference}, "") ;
	  }
	  if ($PACKAGE_LIST[$i]{dir} ne "") {
	    exec_cmd("$UNSHARE $pkgdir");
	  }
	}
    }
  }

#-----------------------------------------------------------
#
#   removing a list of packages
#
#-----------------------------------------------------------

sub remove_packages
  {
    # removed in thre reverse order
    for (my $i = scalar(@PACKAGE_LIST) - 1 ; $i >= 0 ; $i--) {
      if (to_be_processed($PACKAGE_LIST[$i]{phase}, $PACKAGE_LIST[$i]{men},
			  $PACKAGE_LIST[$i]{nmen}, $PACKAGE_LIST[$i]{scope}) eq TRUE) 
	{
	  if ($FOR_MEN eq TRUE) {
	    remove_men_pkg($NODE_ID, $PACKAGE_LIST[$i]{reference}, $SCOPE) ;
	  } 
	  if ($FOR_NMEN eq TRUE) {
	    remove_nmen_pkg($NODE_ID, $PACKAGE_LIST[$i]{reference}) ;
	  }
	}
    }
  }

#-----------------------------------------------------------
# PATCHES
#-----------------------------------------------------------

#-----------------------------------------------------------
#
#   installing a list of patches
#
#-----------------------------------------------------------

sub install_patches
  {
    my $node_ip    ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${NODE_ID}";
    my $server_ip  ="${SUBNET0}.${CLUSTER}.${SUBNET1}.${SERVER_NODE}";

    for ($i = 0 ; $i < scalar(@PATCH_LIST) ; $i++) {
      if (to_be_processed($PATCH_LIST[$i]{phase}, $PATCH_LIST[$i]{men},
			  $PATCH_LIST[$i]{nmen}, $PATCH_LIST[$i]{scope}) eq TRUE) 
	{
	  exec_cmd("$SHARE -F nfs -o ro,anon=0 $PATCH_LIST[$i]{dir}") ||
	    error ( "Unable to NFS share $$PATCH_LIST[$i]{dir}" );   
	  if ($FOR_MEN eq TRUE) {
	    install_men_patch($NODE_ID, $PATCH_LIST[$i]{dir}, $PATCH_LIST[$i]{subdir},
			      $PATCH_LIST[$i]{reference}, $SCOPE) ;
	  } 
	  if ($FOR_NMEN eq TRUE) {
	    install_nmen_patch($NODE_ID, $PATCH_LIST[$i]{dir}, $PATCH_LIST[$i]{subdir},
			      $PATCH_LIST[$i]{reference}, $WHERE) ;
	  }
	  exec_cmd("$UNSHARE $PATCH_LIST[$i]{dir}");
	}
    }
  }

#-----------------------------------------------------------
#
#   removing a list of patches
#
#-----------------------------------------------------------

sub remove_patches
  {
    # removed in thre reverse order
    for (my $i = scalar(@PATCH_LIST) - 1 ; $i >= 0 ; $i--) {
      if (to_be_processed($PATCH_LIST[$i]{phase}, $PATCH_LIST[$i]{men},
			  $PATCH_LIST[$i]{nmen}, $PATCH_LIST[$i]{scope}) eq TRUE) 
	{
	  if ($FOR_MEN eq TRUE) {
	    remove_men_patch($NODE_ID, $PATCH_LIST[$i]{reference}, $SCOPE) ;
	  } 
	  if ($FOR_NMEN eq TRUE) {
	    remove_nmen_patch($NODE_ID, $PATCH_LIST[$i]{reference}, $WHERE) ;
	  }
	}
    }
  }

#-----------------------------------------------------------
#
#   install add-on
#
#-----------------------------------------------------------

sub install_addon
  {
    install_packages();
    install_patches();
  }

#-----------------------------------------------------------
#
#   remove add-on
#
#-----------------------------------------------------------

sub remove_addon
  {  
    remove_patches();
    remove_packages();
  }

##################################################################################
#
#                               This is the main
#
##################################################################################

@required = [ ROOT_DIR, SOLARIS_DIR, NODE_ROLE, WORKING_DIR, PACKAGE_SUBDIR, SOLARIS_SUBDIR, PHASE] ;
init_and_read_arg($0, @required, []) ;

if ($DO_ADDON eq FALSE) {
  exit(0) ;
}

my $name="for unknown phase";
SWITCH: {
  if ($PHASE eq "S_MEN") {
    $name = "after Solaris installation on master-eligible nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "S_NMEN") {
    $name = "after Solaris installation on diskless nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "F_MEN") {
    $name = "after Foundation Services installation on master-eligible nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "F_NMEN") {
    $name = "after Foundation Services installation on diskless nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "A_MEN") {
    $name = "after Application Services installation on master-eligible nodes" ;
    last SWITCH;
  }
  if ($PHASE eq "A_NMEN") {
    $name = "after Application Services installation on diskless nodes" ;
    last SWITCH;
  }
}

if ($RECOVERY ne "") {
  print_recovery_node("Add-on $name: clean-up") ;
  remove_addon();
  
} else {
  
  print_stage_node("Add-on $name") ;
  install_addon();
}

# That's all folk!
exit(0);





